import crypto from 'crypto';
import React, { Component } from 'react';
import PropTypes from 'prop-types';

var isFunction = function isFunction(target) {
  return Object.prototype.toString.call(target) === "[object Function]";
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

if (!supportsIntersectionObserver()) {
  require('intersection-observer');
}

// or add some babel plugins use promise & dynamic-import 👇
// const loadPolyfills = () => {
//   const polyfills = []
//   if(!supportsIntersectionObserver()) polyfills.push(import('intersection-observer')) 
//   return new Promise.all(polyfills)
// }

// loadPolyfills.then(/*...*//)

function supportsIntersectionObserver() {
  var w = window;
  return typeof w !== 'undefined' && 'IntersectionObserver' in w && 'IntersectionObserverEntry' in w && 'isIntersecting' in w.IntersectionObserverEntry.prototype && 'intersectionRatio' in w.IntersectionObserverEntry.prototype;
}

var VPIntersectionObserver = function VPIntersectionObserver() {
  var _this = this;

  var root = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var rootMargin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '-10px 0px';
  classCallCheck(this, VPIntersectionObserver);

  this.handleEntries = function (entries, observer) {
    entries.forEach(function (entry) {
      var isIntersecting = entry.isIntersecting,
          intersectionRatio = entry.intersectionRatio,
          target = entry.target;

      // support IE 10 and under, dataset only works well for IE11

      var observedId = target.getAttribute('data-id');

      var isIntersected = isIntersecting || intersectionRatio > 0;

      // enter
      if (!_this.intersected && isIntersected) {
        _this.intersected = true;
        _this.emitEnter(observedId);
      }

      // leave
      if (_this.intersected && !isIntersected) {
        _this.intersected = false;
        _this.emitLeave(observedId);
      }
    });
  };

  this.emitEnter = function (id) {
    _this.observers.forEach(function (observer) {
      if (observer.id === id) {
        observer.enter();
      }
    });
  };

  this.emitLeave = function (id) {
    _this.observers.forEach(function (observer) {
      if (observer.id === id) {
        observer.leave();
      }
    });
    _this.off(id);
  };

  this.on = function () {
    var observer = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var id = observer.id,
        _observer$enter = observer.enter,
        enter = _observer$enter === undefined ? function () {} : _observer$enter,
        _observer$leave = observer.leave,
        leave = _observer$leave === undefined ? function () {} : _observer$leave,
        target = observer.target;

    if (!id || !target) return;
    if (!isFunction(enter) || !isFunction(leave)) throw new TypeError('the enter or leave callbacks must be a function!');

    if (!!~_this.observers.findIndex(function (observer) {
      return observer.id === id;
    })) return;

    _this.observer.observe(target);
    _this.observers.push(observer);
  };

  this.off = function (id) {

    // unobserve all target in queue
    if (!id) {
      _this.observers.forEach(function (_ref) {
        var target = _ref.target;

        _this.observer.unobserve(target);
      });
      _this.observers = [];
    }

    var idx = _this.observers.findIndex(function (observer) {
      return observer.id === id;
    });
    if (idx < 0) return;

    var _observers$splice = _this.observers.splice(idx, 1),
        target = _observers$splice.target;

    if (!target) return;

    _this.observer.unobserve(target);

    return _this.observers;
  };

  this.getCurrObserver = function () {
    return _this.observer;
  };

  this.destory = function () {
    if (_this.observer) _this.observer.disconnect();
    _this.observer = null;
    _this.observers = [];
  };

  var opts = {
    root: root,
    rootMargin: rootMargin,
    threshold: [0, Number.MIN_VALUE, 0.01]
  };

  if (!this.observer) {
    this.observer = new IntersectionObserver(this.handleEntries, opts);
  }

  this.intersected = false;
  this.observers = [];
};

// Unique ID creation requires a high quality random # generator.  In node.js
// this is pretty straight-forward - we use the crypto API.



var rng = function nodeRNG() {
  return crypto.randomBytes(16);
};

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */
var byteToHex = [];
for (var i = 0; i < 256; ++i) {
  byteToHex[i] = (i + 0x100).toString(16).substr(1);
}

function bytesToUuid(buf, offset) {
  var i = offset || 0;
  var bth = byteToHex;
  // join used to fix memory issue caused by concatenation: https://bugs.chromium.org/p/v8/issues/detail?id=3175#c4
  return ([bth[buf[i++]], bth[buf[i++]], 
	bth[buf[i++]], bth[buf[i++]], '-',
	bth[buf[i++]], bth[buf[i++]], '-',
	bth[buf[i++]], bth[buf[i++]], '-',
	bth[buf[i++]], bth[buf[i++]], '-',
	bth[buf[i++]], bth[buf[i++]],
	bth[buf[i++]], bth[buf[i++]],
	bth[buf[i++]], bth[buf[i++]]]).join('');
}

var bytesToUuid_1 = bytesToUuid;

function v4(options, buf, offset) {
  var i = buf && offset || 0;

  if (typeof(options) == 'string') {
    buf = options === 'binary' ? new Array(16) : null;
    options = null;
  }
  options = options || {};

  var rnds = options.random || (options.rng || rng)();

  // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
  rnds[6] = (rnds[6] & 0x0f) | 0x40;
  rnds[8] = (rnds[8] & 0x3f) | 0x80;

  // Copy bytes to buffer, if provided
  if (buf) {
    for (var ii = 0; ii < 16; ++ii) {
      buf[i + ii] = rnds[ii];
    }
  }

  return buf || bytesToUuid_1(rnds);
}

var v4_1 = v4;

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = "/* add css styles here (optional) */\n\n.styles_wrapper__3KXDn {\n  overflow: hidden;\n  position: relative;\n  width: 100%;\n  height: 100%;\n  background-size: 100% 100%;\n}\n\n.styles_baseLayout__3rQPq {\n  position: absolute;\n  top: 0;\n  bottom: 0;\n  left: 0;\n  right: 0;\n  display: block;\n\n  width: 100%;\n  height: 100%;\n}\n\n.styles_image__24T34 {\n\n  z-index: 2;\n  opacity: 0;\n  transition: opacity 1s ease-in;\n}\n\n.styles_placeholder__3Mrh_ {\n}\n\n.styles_imageFadeIn__19LY0 {\n  opacity: 1;\n}\n\n\n\n\n";
var styles = { "wrapper": "styles_wrapper__3KXDn", "baseLayout": "styles_baseLayout__3rQPq", "image": "styles_image__24T34 styles_baseLayout__3rQPq", "placeholder": "styles_placeholder__3Mrh_ styles_baseLayout__3rQPq", "imageFadeIn": "styles_imageFadeIn__19LY0" };
styleInject(css);

var BetterImage = function (_Component) {
  inherits(BetterImage, _Component);

  function BetterImage(props) {
    classCallCheck(this, BetterImage);

    var _this = possibleConstructorReturn(this, (BetterImage.__proto__ || Object.getPrototypeOf(BetterImage)).call(this, props));

    _this.imgOnLoad = function () {
      var onload = _this.props.onload;

      _this.setState({ imageStyle: '' + styles.imageFadeIn });
      if (onload) onload();
    };

    _this.renderImage = function () {
      var _this$props = _this.props,
          alt = _this$props.alt,
          source = _this$props.source;
      var _this$state = _this.state,
          imageStyle = _this$state.imageStyle,
          startLoad = _this$state.startLoad;

      if (!startLoad) return;
      return React.createElement('img', {
        src: source,
        onLoad: _this.imgOnLoad,
        className: styles.image + ' ' + imageStyle,
        alt: alt
      });
    };

    _this.renderPlaceholder = function () {
      var _this$props2 = _this.props,
          placeholder = _this$props2.placeholder,
          alt = _this$props2.alt;


      if (placeholder && isFunction(placeholder)) {
        return React.createElement(
          React.Fragment,
          null,
          placeholder()
        );
      }

      if (placeholder && React.isValidElement(placeholder)) {
        return React.createElement(
          React.Fragment,
          null,
          placeholder
        );
      }

      return React.createElement('img', {
        src: placeholder,
        alt: alt,
        className: styles.placeholder
      });
    };

    _this.state = {
      id: v4_1(),
      imageStyle: '',
      startLoad: false
    };
    _this.oberserver = null;
    return _this;
  }

  createClass(BetterImage, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      var id = this.state.id;
      var _props = this.props,
          _enter = _props.enter,
          _leave = _props.leave,
          onlyEnter = _props.onlyEnter,
          root = _props.root,
          rootMargin = _props.rootMargin;
      var target = this.target;


      if (!this.oberserver) this.oberserver = new VPIntersectionObserver(root, rootMargin);

      this.oberserver.on({
        id: id,
        target: target,
        enter: function enter(target) {
          _this2.setState({ startLoad: true });
          if (_enter) _enter(target);
          if (onlyEnter) _this2.oberserver.off(id);
        },
        leave: function leave(target) {
          if (_leave) _leave(target);
        }
      });
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.oberserver) this.oberserver.destory();
      if (this.target) this.target = null;
      this.oberserver = null;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var id = this.state.id;

      return React.createElement(
        'div',
        {
          ref: function ref(currRef) {
            return _this3.target = currRef;
          },
          'data-id': id,
          className: styles.wrapper
        },
        this.renderPlaceholder(),
        this.renderImage()
      );
    }
  }]);
  return BetterImage;
}(Component);

BetterImage.propTypes = {
  source: PropTypes.string.isRequired,
  placeholder: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.element]).isRequired,
  alt: PropTypes.string,
  onload: PropTypes.func,
  enter: PropTypes.func,
  leave: PropTypes.func,
  onlyEnter: PropTypes.bool,
  root: PropTypes.node,
  rootMargin: PropTypes.string
};

BetterImage.defaultProps = {
  alt: '',
  onlyEnter: true
};

export default BetterImage;
//# sourceMappingURL=index.es.js.map
